#!/bin/sh

# ============================================================================
# Dameng ODBC Driver Installer for Linux
# ============================================================================
# Description: This script installs the Dameng ODBC driver files to a chosen
#              destination and configures the system linker and odbcinst.ini.
# ============================================================================

# --- Configuration Constants ---
LIB_NAME="libdodbc.so"
CONF_FILE="/etc/ld.so.conf.d/libdodbc.conf"
ODBC_INST_FILE="/etc/odbcinst.ini"
DRIVER_SECTION="DM8 ODBC DRIVER"
DEFAULT_DEST_PATH="/opt/dameng_odbc"
ENV_VAR_FILE="/etc/profile.d/dm_home.sh"

# --- Help Message ---
show_help() {
  echo "Usage: $0"
  echo "Configure and install the ODBC driver for the Dameng database."
  echo
  echo "This script must be run from the directory containing the Dameng"
  echo "ODBC driver files (e.g., libdodbc.so)."
  echo
  echo "Options:"
  echo "  -h, --help         Show this help message"
  exit 0
}

# --- Argument Parser ---
if [ "$1" = "-h" ] || [ "$1" = "--help" ]; then
  show_help
fi

# --- Pre-run Checks ---
# Check for required tools
REQUIRED_TOOLS="grep awk sed sudo ldconfig cp mkdir dirname chmod tee"
MISSING_TOOLS=""

for tool in $REQUIRED_TOOLS; do
  if ! command -v "$tool" >/dev/null 2>&1; then
    MISSING_TOOLS="$MISSING_TOOLS $tool"
  fi
done

if [ -n "$MISSING_TOOLS" ]; then
  echo "ERROR: Missing required tools:"
  for t in $MISSING_TOOLS; do
    echo "  - $t"
  done
  exit 1
fi

# Check if running with sudo privileges already
if [ "$(id -u)" -ne 0 ]; then
    echo "This script requires superuser privileges for installation."
    echo "You will be prompted for your password by 'sudo'."
    echo
fi

# --- Main Execution ---

# Get the directory where the script and driver files are located
SCRIPT_DIR=$(cd "$(dirname "$0")" && pwd)
if [ ! -f "${SCRIPT_DIR}/${LIB_NAME}" ]; then
    echo "ERROR: The required driver file '${LIB_NAME}' was not found in the script's directory."
    echo "Please run this script from the same folder where the Dameng driver files are located."
    exit 1
fi

# ============================================================================
# Phase 1: Copy Files to Destination
# ============================================================================
echo "--- Phase 1: File Installation ---"
echo
echo "Please specify the installation destination for the Dameng ODBC driver."
printf "Enter path (or press Enter to use default: %s): " "$DEFAULT_DEST_PATH"
read -r USER_PATH

# Set destination path to default if user input is empty
if [ -z "$USER_PATH" ]; then
  DEST_PATH="$DEFAULT_DEST_PATH"
else
  DEST_PATH="$USER_PATH"
fi

echo "Driver will be installed to: ${DEST_PATH}"
echo

# Create destination directory
echo "Creating destination directory (requires sudo)..."
sudo mkdir -p "$DEST_PATH"
if [ $? -ne 0 ]; then
    echo "ERROR: Failed to create destination directory. Please check permissions."
    exit 1
fi

# Copy all files from the source directory except the script itself
echo "Copying driver files..."
# The find command is a safe way to copy all contents of the current directory
# (including hidden files and subdirectories) except for this script itself.
sudo find "$SCRIPT_DIR" -mindepth 1 -maxdepth 1 -not -name "$(basename "$0")" -exec cp -r {} "$DEST_PATH/" \;
if [ $? -ne 0 ]; then
    echo "ERROR: Failed to copy files to the destination."
    exit 1
fi
echo "Files copied successfully."
echo

# ============================================================================
# Phase 2: System Configuration
# ============================================================================
echo "--- Phase 2: System Configuration ---"
echo

# Verify the main library exists in the destination before proceeding
if [ ! -f "${DEST_PATH}/${LIB_NAME}" ]; then
  echo "ERROR: Critical library '${LIB_NAME}' not found in destination after copy."
  echo "Installation cannot continue."
  exit 1
fi

# --- Linker Configuration ---
echo "Updating linker configuration..."
# Create a new conf file pointing to the library path
echo "$DEST_PATH" | sudo tee "$CONF_FILE" >/dev/null
# Update the linker cache
sudo ldconfig
echo "Linker configuration updated."
echo

# --- ODBC Driver Registration ---
echo "Updating ODBC driver registration in ${ODBC_INST_FILE}..."
# Use awk for safe removal of a previous entry to prevent duplicates
if grep -q "^\[${DRIVER_SECTION}\]" "$ODBC_INST_FILE" 2>/dev/null; then
  echo "A previous entry was found. It will be replaced."
  TEMP_FILE=$(mktemp)
  sudo awk -v section="[${DRIVER_SECTION}]" '
    BEGIN { in_section = 0 }
    # If we find a line that starts a new section
    $0 ~ /^\[.*\]/ {
        # If it is our section, start skipping lines
        if ($0 == section) {
            in_section = 1
        } else {
            # Otherwise, stop skipping and print the line
            in_section = 0
            print
        }
        next
    }
    # If we are not in the section, print the line
    !in_section { print }
  ' "$ODBC_INST_FILE" > "$TEMP_FILE" && sudo mv "$TEMP_FILE" "$ODBC_INST_FILE"
fi

# Add the new, correct entry to odbcinst.ini
DRIVER_FULL_PATH="${DEST_PATH}/${LIB_NAME}"
{
  printf "\n[%s]\n" "$DRIVER_SECTION"
  printf "Description = Dameng ODBC Driver\n"
  printf "Driver = %s\n" "$DRIVER_FULL_PATH"
} | sudo tee -a "$ODBC_INST_FILE" >/dev/null

# --- Validation ---
if grep -q "^Driver = ${DRIVER_FULL_PATH}" "$ODBC_INST_FILE"; then
  echo "Successfully registered ODBC driver."
  grep -A 2 "^\[${DRIVER_SECTION}\]" "$ODBC_INST_FILE"
else
  echo "ERROR: Failed to register ODBC driver in ${ODBC_INST_FILE}!"
  exit 1
fi
echo

# ============================================================================
# Phase 2a: Set Environment Variable
# ============================================================================
echo "--- Phase 2a: Setting DM_HOME Environment Variable ---"
echo

# First, check if the variable is already set in the current environment
if [ -n "$DM_HOME" ]; then
    DM_HOME_ACTION="existed"
    EXISTING_VALUE="$DM_HOME"
    echo "Environment variable 'DM_HOME' is already set in the current session."
    echo "Value: $EXISTING_VALUE"
    echo "Skipping this step."
else
    # If not in current session, check persistent system-wide profile files
    EXISTING_SETTING=$(grep -r "^export DM_HOME=" /etc/profile.d/ /etc/environment 2>/dev/null | head -n 1)
    if [ -n "$EXISTING_SETTING" ]; then
        DM_HOME_ACTION="existed"
        EXISTING_VALUE=$(echo "$EXISTING_SETTING" | cut -d'=' -f2)
        echo "System environment variable 'DM_HOME' appears to be already set in configuration files."
        echo "Found setting: $EXISTING_VALUE"
        echo "Skipping this step to avoid conflicts."
    else
        # If it doesn't exist anywhere, set it.
        echo "Setting system environment variable 'DM_HOME' to '${DEST_PATH}'..."
        # Create a script in /etc/profile.d to set the variable for all users on login
        echo "export DM_HOME=\"${DEST_PATH}\"" | sudo tee "${ENV_VAR_FILE}" >/dev/null
        sudo chmod +x "${ENV_VAR_FILE}"
        if [ $? -eq 0 ]; then
            DM_HOME_ACTION="set"
            echo "Successfully created ${ENV_VAR_FILE}."
            echo "The 'DM_HOME' variable will be available in new terminal sessions or after a reboot."
        else
            DM_HOME_ACTION="failed"
            echo "ERROR: Failed to create environment variable file."
        fi
    fi
fi
echo

# ============================================================================
# Phase 3: Installation Summary
# ============================================================================
echo "--- Phase 3: Installation Summary ---"
echo
echo "==================================================="
echo "  Dameng DM8 ODBC Driver Installation Complete"
echo "==================================================="
echo
printf "  %-22s %s\n" "Driver Name:" "$DRIVER_SECTION"
printf "  %-22s %s\n" "Installed To:" "$DEST_PATH"

if [ "$DM_HOME_ACTION" = "set" ]; then
    printf "  %-22s %s\n" "Environment Variable:" "DM_HOME set to ${DEST_PATH}"
elif [ "$DM_HOME_ACTION" = "existed" ]; then
    printf "  %-22s %s\n" "Environment Variable:" "DM_HOME already exists (Value: ${EXISTING_VALUE})"
fi

echo
echo "  You can now connect to Dameng Database in Navicat."
echo
echo "==================================================="
echo
